package com.yunzhi.training.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.training.common.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.training.service.ITaCourseService;
import com.yunzhi.training.entity.TaCourse;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;

/**
 * <p>
    * 课程表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-02-21
 */

@Api(tags = "课程表")
@RestController
@RequestMapping("/")
public class TaCourseController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaCourseController.class);

    @Autowired
    public ITaCourseService iTaCourseService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/course",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taCourseList(@ApiParam(value = "客户端",allowableValues = "admin,wx") @PathVariable String client,
                                     @ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                     @ApiParam("名称") @RequestParam(value ="title", required = false) String title,
                                     @ApiParam(value = "开始时间", example = "2022-02-21") @RequestParam(value ="startDate", required = false) String startDate,
                                     @ApiParam(value = "结束时间", example = "2022-02-28") @RequestParam(value ="endDate", required = false) String endDate) throws Exception{
        LocalDateTime start = null;
        LocalDateTime end = null;
        if ((StringUtils.isEmpty(startDate) && !StringUtils.isEmpty(endDate)) || (!StringUtils.isEmpty(startDate) && StringUtils.isEmpty(endDate))) {
            return ResponseBean.error("请选择完整的开始结束时间", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        IPage<TaCourse> pg = new Page<>(pageNum, pageSize);

        if (!StringUtils.isEmpty(startDate)) {
            start = DateUtils.from(startDate + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
            end = DateUtils.from(endDate + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }

        IPage<TaCourse> result = iTaCourseService.getPagedBy(pg, client, title, start, end);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taCourse 实体对象
     * @return
     */
    @RequestMapping(value="/admin/course",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taCourseAdd(@ApiParam("保存内容") @RequestBody TaCourse taCourse) throws Exception{

        if (StringUtils.isEmpty(taCourse.getTitle())) {
            return ResponseBean.error("课程名称不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        int cnt = iTaCourseService.countBy("title", taCourse.getTitle(), true);
        if (cnt > 0) {
            return ResponseBean.error("课程名称重复", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        taCourse.setCourseId(null);

        if (iTaCourseService.save(taCourse)){
            return ResponseBean.success(taCourse);
        } else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/course/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taCourseDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        if(iTaCourseService.removeLogicById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taCourse 实体对象
     * @return
     */
    @RequestMapping(value="/admin/course/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taCourseUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TaCourse taCourse) throws Exception{

        if (StringUtils.isEmpty(taCourse.getTitle())) {
            return ResponseBean.error("课程名称不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TaCourse origin = iTaCourseService.getById(id);
        if (null == origin || Constants.STATUS_DELETED == origin.getState()) {
            return ResponseBean.error("更新失败, 未找到有效资源", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TaCourse exist = iTaCourseService.getByButNot("title", taCourse.getTitle(), "course_id", id, true);
        if (exist != null) {
            return ResponseBean.error("课程称重复", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (iTaCourseService.updateById(taCourse)){
            return ResponseBean.success(iTaCourseService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/course/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taCourseGet(@ApiParam(value = "客户端",allowableValues = "admin,wx") @PathVariable String client,
                                    @ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(iTaCourseService.getExistBy("course_id", id, "wx".equals(client), true));
    }
}
