package com.yunzhi.training.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.training.common.BaseController;
import com.yunzhi.training.common.Constants;
import com.yunzhi.training.common.ResponseBean;
import com.yunzhi.training.common.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.training.service.ITaSchoolClassService;
import com.yunzhi.training.entity.TaSchoolClass;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;

/**
 * <p>
    * 班级 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-02-21
 */

@Api(tags = "班级")
@RestController
@RequestMapping("/")
public class TaSchoolClassController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaSchoolClassController.class);

    @Autowired
    public ITaSchoolClassService iTaSchoolClassService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/school-class",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taSchoolClassList(@ApiParam(value = "客户端",allowableValues = "admin,wx") @PathVariable String client,
                                          @ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                          @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                          @ApiParam("所属学期") @RequestParam(value ="termId", required = false) String termId,
                                          @ApiParam("名称") @RequestParam(value ="name", required = false) String name) throws Exception{

        IPage<TaSchoolClass> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaSchoolClass> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(!StringUtils.isEmpty(termId), "term_id", termId);
        queryWrapper.like(!StringUtils.isEmpty(name), "name", "%"+name+"%");
        queryWrapper.gt("admin".equals(client), "state", Constants.STATUS_DELETED);
        queryWrapper.eq("wx".equals(client), "state", Constants.STATUS_NORMAL);
        queryWrapper.orderByDesc("create_date");

        IPage<TaSchoolClass> result = iTaSchoolClassService.getPageBy(pg, termId, name, client);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taSchoolClass 实体对象
     * @return
     */
    @RequestMapping(value="/admin/school-class",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taSchoolClassAdd(@ApiParam("保存内容") @RequestBody TaSchoolClass taSchoolClass) throws Exception{

        if (StringUtils.isEmpty(taSchoolClass.getTermId())) {
            return ResponseBean.error("请设置班级所属学期", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (StringUtils.isEmpty(taSchoolClass.getName())) {
            return ResponseBean.error("班级名称不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        int cnt = iTaSchoolClassService.countName(taSchoolClass.getTermId(), taSchoolClass.getName());
        if (cnt > 0) {
            return ResponseBean.error("班级名称重复", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        taSchoolClass.setClassId(null);
        taSchoolClass.setCreateDate(LocalDateTime.now());

        if (iTaSchoolClassService.save(taSchoolClass)){
            return ResponseBean.success(taSchoolClass);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/school-class/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taSchoolClassDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        if(iTaSchoolClassService.removeLogicById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taSchoolClass 实体对象
     * @return
     */
    @RequestMapping(value="/admin/school-class/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taSchoolClassUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TaSchoolClass taSchoolClass) throws Exception{
        if (StringUtils.isEmpty(taSchoolClass.getTermId())) {
            return ResponseBean.error("请设置班级所属学期", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (StringUtils.isEmpty(taSchoolClass.getName())) {
            return ResponseBean.error("班级名称不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TaSchoolClass origin = iTaSchoolClassService.getById(id);
        if (null == origin || Constants.STATUS_DELETED == origin.getState()) {
            return ResponseBean.error("更新失败, 未找到有效资源", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        taSchoolClass.setClassId(id);
        boolean isOnly = iTaSchoolClassService.checkOnly(taSchoolClass);
        if (!isOnly) {
            return ResponseBean.error("班级名称重复", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        origin.setName(taSchoolClass.getName());
        origin.setTermId(taSchoolClass.getTermId());
        origin.setState(taSchoolClass.getState());

        if (iTaSchoolClassService.updateById(origin)){
            return ResponseBean.success(origin);
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/school-class/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taSchoolClassGet(@ApiParam(value = "客户端",allowableValues = "admin,wx") @PathVariable String client,
                                         @ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(iTaSchoolClassService.getExistBy("class_id", id, "wx".equals(client), true));
    }
}
